ERROR_CODE = {
    # 通用
    0: "操作成功",
    1: "操作全部失败",
    2: "批量操作部分成功",
    50000: "body不能为空",
    50001: "撮合引擎正在升级，请稍后重试",
    50002: "非法的json数据",
    50004: "接口请求超时（不代表请求成功或者失败，请检查请求结果）",
    50005: "接口已下线或无法使用",
    50006: "无效的Content_Type，请使用application/json格式",
    50007: "用户被冻结",
    50008: "用户不存在",
    50009: "用户处于爆仓冻结",
    50010: "用户ID为空",
    50011: "用户请求频率过快，超过该接口允许的限额",
    50012: "用户状态无效",
    50013: "当前系统繁忙，请稍后重试",
    50014: "必填参数{0}不能为空",
    50015: "参数{0}和{1}不能同时为空",
    50016: "参数{0}和{1}不匹配",
    50017: "当前仓位处于ADL冻结中，无法进行相关操作",
    50018: "{0}币种处于ADL冻结中，无法进行相关操作",
    50019: "当前账户处于ADL冻结中，无法进行相关操作",
    50020: "当前仓位处于强平冻结中，无法进行相关操作",
    50021: "{0}币种处于强平冻结中，无法进行相关操作",
    50022: "当前账户处于强平冻结中，无法进行相关操作",
    50023: "资金费冻结，无法进行相关操作",
    50024: "参数{0}和{1}不能同时存在",
    50025: "参数{0}传值个数超过最大限制{1}",
    50026: "系统错误",
    50027: "当前账户已被限制交易",
    50028: "账户异常无法下单",
    50029: "你的账户已经触发风控体系，禁止该标的{0}交易，请联系客服进行处理",
    # API 类
    # 交易类
    51023: "仓位不存在"
}